@@ -63,14 +63,20 @@ import com.google.api.services.drive.model.ChildList;
 import com.google.api.services.drive.model.ChildReference;
 import com.google.api.services.drive.model.File;
 import com.google.api.services.drive.model.ParentReference;
+import com.openexchange.config.cascade.ConfigView;
+import com.openexchange.config.cascade.ConfigViewFactory;
+import com.openexchange.config.cascade.ConfigViews;
 import com.openexchange.exception.OXException;
 import com.openexchange.file.storage.FileStorageAccount;
+import com.openexchange.file.storage.FileStorageAutoRenameFoldersAccess;
 import com.openexchange.file.storage.FileStorageExceptionCodes;
 import com.openexchange.file.storage.FileStorageFolder;
 import com.openexchange.file.storage.FileStorageFolderAccess;
+import com.openexchange.file.storage.NameBuilder;
 import com.openexchange.file.storage.Quota;
 import com.openexchange.file.storage.Quota.Type;
-import com.openexchange.file.storage.googledrive.access.GoogleDriveAccess;
+import com.openexchange.file.storage.googledrive.access.GoogleDriveOAuthAccess;
+import com.openexchange.file.storage.googledrive.osgi.Services;
 import com.openexchange.session.Session;
 
 /**
@@ -78,7 +84,7 @@ import com.openexchange.session.Session;
  *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess implements FileStorageFolderAccess {
+public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess implements FileStorageFolderAccess, FileStorageAutoRenameFoldersAccess {
 
     private static final String QUERY_STRING_DIRECTORIES_ONLY = GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY;
 
@@ -86,14 +92,19 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private final int userId;
     private final String accountDisplayName;
+    private final boolean useOptimisticSubfolderDetection;
 
     /**
      * Initializes a new {@link GoogleDriveFolderAccess}.
      */
-    public GoogleDriveFolderAccess(final GoogleDriveAccess googleDriveAccess, final FileStorageAccount account, final Session session) throws OXException {
+    public GoogleDriveFolderAccess(final GoogleDriveOAuthAccess googleDriveAccess, final FileStorageAccount account, final Session session) throws OXException {
         super(googleDriveAccess, account, session);
         userId = session.getUserId();
         accountDisplayName = account.getDisplayName();
+        ConfigViewFactory viewFactory = Services.optService(ConfigViewFactory.class);
+        ConfigView view = viewFactory.getView(session.getUserId(), session.getContextId());
+        useOptimisticSubfolderDetection = ConfigViews.getDefinedBoolPropertyFrom("com.openexchange.file.storage.googledrive.useOptimisticSubfolderDetection", true, view);
+
     }
 
     private void checkDirValidity(com.google.api.services.drive.model.File file) throws OXException {
@@ -104,7 +115,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
     }
 
     private GoogleDriveFolder parseGoogleDriveFolder(com.google.api.services.drive.model.File dir, Drive drive) throws OXException, IOException {
-        return new GoogleDriveFolder(userId).parseDirEntry(dir, getRootFolderId(), accountDisplayName, drive);
+        return new GoogleDriveFolder(userId).parseDirEntry(dir, getRootFolderId(), accountDisplayName, useOptimisticSubfolderDetection, drive);
     }
 
     @Override
@@ -114,7 +125,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private boolean exists(String folderId, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
             com.google.api.services.drive.model.File file = drive.files().get(toGoogleDriveFolderId(folderId)).execute();
             Boolean explicitlyTrashed = file.getExplicitlyTrashed();
             return isDir(file) && (null == explicitlyTrashed || !explicitlyTrashed.booleanValue());
@@ -134,7 +145,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(null, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return exists(folderId, retry);
         } catch (final IOException e) {
@@ -151,7 +162,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private FileStorageFolder getFolder(String folderId, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
             com.google.api.services.drive.model.File dir = drive.files().get(toGoogleDriveFolderId(folderId)).execute();
             checkDirValidity(dir);
             return parseGoogleDriveFolder(dir, drive);
@@ -168,7 +179,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return getFolder(folderId, retry);
         } catch (final IOException e) {
@@ -205,7 +216,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private FileStorageFolder[] getSubfolders(String parentIdentifier, boolean all, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
 
             Drive.Children.List list = drive.children().list(toGoogleDriveFolderId(parentIdentifier));
             list.setQ(GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
@@ -247,7 +258,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(parentIdentifier, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return getSubfolders(parentIdentifier, all, retry);
         } catch (final IOException e) {
@@ -271,28 +282,65 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     @Override
     public String createFolder(FileStorageFolder toCreate) throws OXException {
-        return createFolder(toCreate, 0);
+        return createFolder(toCreate, true);
     }
 
-    private String createFolder(FileStorageFolder toCreate, int retryCount) throws OXException {
+    @Override
+    public String createFolder(FileStorageFolder toCreate, boolean autoRename) throws OXException {
+        return createFolder(toCreate, autoRename, 0);
+    }
+
+    private String createFolder(FileStorageFolder toCreate, boolean autorename, int retryCount) throws OXException {
         String parentId = toGoogleDriveFolderId(toCreate.getParentId());
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive> getClient().client;
+            String baseName = toCreate.getName();
 
-            Drive.Children.List list = drive.children().list(parentId);
-            list.setQ("title='"+toCreate.getName()+"' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
-            if (!list.execute().getItems().isEmpty()) {
-                // Already such a folder
-                throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(toCreate.getName(), drive.files().get(parentId).execute().getTitle());
+            NameBuilder name = new NameBuilder(baseName);
+            if (autorename) {
+                // Duplicate name needs to be explicitly checked since Google Drive allows multiple folders with the same name next to each other
+                {
+                    Drive.Children.List list = drive.children().list(parentId);
+                    list.setQ(GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
+                    List<ChildReference> existingFolders = list.execute().getItems();
+                    if (!existingFolders.isEmpty()) {
+                        // Check if there is already such a folder
+                        boolean alreadySuchAFolder;
+                        do {
+                            alreadySuchAFolder = false;
+                            String nameToCheck = name.toString();
+                            for (ChildReference childReference : existingFolders) {
+                                if (drive.files().get(childReference.getId()).execute().getTitle().equals(nameToCheck)) {
+                                    name.advance();
+                                    alreadySuchAFolder = true;
+                                    break;
+                                }
+                            }
+                        } while (alreadySuchAFolder);
+                    }
+                }
+            } else {
+                String nameToCheck = name.toString();
+                Drive.Children.List list = drive.children().list(parentId);
+                list.setQ("title='" + nameToCheck + "' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
+                List<ChildReference> existingFolders = list.execute().getItems();
+                if (!existingFolders.isEmpty()) {
+                    // Check if there is already such a folder
+                    for (ChildReference childReference : existingFolders) {
+                        if (drive.files().get(childReference.getId()).execute().getTitle().equals(nameToCheck)) {
+                            throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(nameToCheck, drive.files().get(parentId).execute().getTitle());
+                        }
+                    }
+                }
             }
 
+
             File driveDir = new File();
             driveDir.setParents(Collections.singletonList(new ParentReference().setId(parentId)));
-            driveDir.setTitle(toCreate.getName());
+            driveDir.setTitle(name.toString());
             driveDir.setMimeType(GoogleDriveConstants.MIME_TYPE_DIRECTORY);
 
             File newDir = drive.files().insert(driveDir).execute();
-
             return toFileStorageFolderId(newDir.getId());
         } catch (final HttpResponseException e) {
             if (!isUserRateLimitExceeded(e)) {
@@ -307,9 +355,9 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(toCreate.getParentId(), e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
-            return createFolder(toCreate, retry);
+            return createFolder(toCreate, autorename, retry);
         } catch (final IOException e) {
             throw FileStorageExceptionCodes.IO_ERROR.create(e, e.getMessage());
         } catch (final RuntimeException e) {
@@ -330,30 +378,63 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     @Override
     public String moveFolder(String folderId, String newParentId, String newName) throws OXException {
-        return moveFolder(folderId, newParentId, newName, 0);
+        return moveFolder(folderId, newParentId, newName, true);
     }
 
-    private String moveFolder(String folderId, String newParentId, String newName, int retryCount) throws OXException {
+    @Override
+    public String moveFolder(String folderId, String newParentId, String newName, boolean autoRename) throws OXException {
+        return moveFolder(folderId, newParentId, newName, autoRename, 0);
+    }
+
+    private String moveFolder(String folderId, String newParentId, String newName, boolean autoRename, int retryCount) throws OXException {
         String fid = toGoogleDriveFolderId(folderId);
         String nfid = toGoogleDriveFolderId(newParentId);
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
 
-            String title = null == newName ? drive.files().get(fid).execute().getTitle() : newName;
+            String baseName = null == newName ? drive.files().get(fid).execute().getTitle() : newName;
 
-            Drive.Children.List list = drive.children().list(nfid);
-            list.setQ("title='"+title+"' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
-            if (!list.execute().getItems().isEmpty()) {
-                // Already such a folder
-                throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(title, drive.files().get(nfid).execute().getTitle());
+            // Duplicate name needs to be explicitly checked since Google Drive allows multiple folders with the same name next to each other
+            NameBuilder name = new NameBuilder(baseName);
+            if (autoRename) {
+                Drive.Children.List list = drive.children().list(nfid);
+                list.setQ(GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
+                List<ChildReference> existingFolders = list.execute().getItems();
+                if (!existingFolders.isEmpty()) {
+                    // Check if there is already such a folder
+                    int retries = 0;
+                    boolean alreadySuchAFolder;
+                    do {
+                        alreadySuchAFolder = false;
+                        String nameToCheck = name.toString();
+                        for (ChildReference childReference : existingFolders) {
+                            if (drive.files().get(childReference.getId()).execute().getTitle().equals(nameToCheck)) {
+                                name.advance();
+                                alreadySuchAFolder = true;
+                                break;
+                            }
+                        }
+                    } while (alreadySuchAFolder);
+                }
+            } else {
+                String nameToCheck = name.toString();
+                Drive.Children.List list = drive.children().list(nfid);
+                list.setQ("title='" + nameToCheck + "' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
+                List<ChildReference> existingFolders = list.execute().getItems();
+                if (!existingFolders.isEmpty()) {
+                    // Check if there is already such a folder
+                    for (ChildReference childReference : existingFolders) {
+                        if (drive.files().get(childReference.getId()).execute().getTitle().equals(nameToCheck)) {
+                            throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(nameToCheck, drive.files().get(nfid).execute().getTitle());
+                        }
+                    }
+                }
             }
 
             File driveDir = new File();
             driveDir.setId(fid);
             driveDir.setParents(Collections.singletonList(new ParentReference().setId(nfid)));
-            if (null != newName) {
-                driveDir.setTitle(newName);
-            }
+            driveDir.setTitle(name.toString());
             driveDir.setMimeType(GoogleDriveConstants.MIME_TYPE_DIRECTORY);
 
             File movedDir = drive.files().patch(fid, driveDir).execute();
@@ -371,9 +452,9 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
-            return moveFolder(folderId, newParentId, newName, retry);
+            return moveFolder(folderId, newParentId, newName, autoRename, retry);
         } catch (final IOException e) {
             throw FileStorageExceptionCodes.IO_ERROR.create(e, e.getMessage());
         } catch (final RuntimeException e) {
@@ -389,7 +470,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
     private String renameFolder(String folderId, String newName, int retryCount) throws OXException {
         String fid = toGoogleDriveFolderId(folderId);
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
             /*
              * get folder to rename
              */
@@ -403,9 +484,15 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
             List<ParentReference> parentReferences = folder.getParents();
             for (ParentReference parentReference : parentReferences) {
                 Drive.Children.List list = drive.children().list(parentReference.getId());
-                list.setQ("title='"+newName+"' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
-                if (false == list.execute().getItems().isEmpty()) {
-                    throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(newName, drive.files().get(parentReference.getId()).execute().getTitle());
+                list.setQ("title='" + newName + "' and " + GoogleDriveConstants.QUERY_STRING_DIRECTORIES_ONLY_EXCLUDING_TRASH);
+                List<ChildReference> existingFolders = list.execute().getItems();
+                if (!existingFolders.isEmpty()) {
+                    // Check if there is already such a folder
+                    for (ChildReference childReference : existingFolders) {
+                        if (drive.files().get(childReference.getId()).execute().getTitle().equals(newName)) {
+                            throw FileStorageExceptionCodes.DUPLICATE_FOLDER.create(newName, drive.files().get(parentReference.getId()).execute().getTitle());
+                        }
+                    }
                 }
             }
             /*
@@ -427,7 +514,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return renameFolder(folderId, newName, retry);
         } catch (final IOException e) {
@@ -449,7 +536,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private String deleteFolder(String folderId, boolean hardDelete, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive> getClient().client;
 
             String fid = toGoogleDriveFolderId(folderId);
             if (hardDelete || isTrashed(fid, drive)) {
@@ -476,7 +563,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return deleteFolder(folderId, hardDelete, retry);
         } catch (final IOException e) {
@@ -498,14 +585,13 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private void clearFolder(String folderId, boolean hardDelete, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive> getClient().client;
             /*
              * build request to list all files in a folder
              */
             String fid = toGoogleDriveFolderId(folderId);
             boolean deletePermanently = hardDelete || isTrashed(fid, drive);
-            com.google.api.services.drive.Drive.Children.List listRequest = drive.children().list(fid)
-                .setQ(GoogleDriveConstants.QUERY_STRING_FILES_ONLY).setFields("nextPageToken,items(id)");
+            com.google.api.services.drive.Drive.Children.List listRequest = drive.children().list(fid).setQ(GoogleDriveConstants.QUERY_STRING_FILES_ONLY).setFields("nextPageToken,items(id)");
             /*
              * execute as often as needed & delete files
              */
@@ -534,7 +620,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             clearFolder(folderId, hardDelete, retry);
             return;
@@ -552,7 +638,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private FileStorageFolder[] getPath2DefaultFolder(String folderId, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
 
             List<FileStorageFolder> list = new LinkedList<FileStorageFolder>();
 
@@ -583,7 +669,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return getPath2DefaultFolder(folderId, retry);
         } catch (final IOException e) {
@@ -600,7 +686,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
 
     private Quota getStorageQuota(String folderId, int retryCount) throws OXException {
         try {
-            Drive drive = googleDriveAccess.getDrive(session);
+            Drive drive = googleDriveAccess.<Drive>getClient().client;
             About about = drive.about().get().setFields("quotaType,quotaBytesUsed,quotaBytesTotal").execute();
             if ("UNLIMITED".equals(about.getQuotaType())) {
                 return Type.STORAGE.getUnlimited();
@@ -619,7 +705,7 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
                 throw handleHttpResponseError(folderId, e);
             }
 
-            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long)(Math.random() * 1000)), TimeUnit.MILLISECONDS);
+            long nanosToWait = TimeUnit.NANOSECONDS.convert((retry * 1000) + ((long) (Math.random() * 1000)), TimeUnit.MILLISECONDS);
             LockSupport.parkNanos(nanosToWait);
             return getStorageQuota(folderId, retry);
         } catch (IOException e) {
@@ -642,14 +728,14 @@ public final class GoogleDriveFolderAccess extends AbstractGoogleDriveAccess imp
         Quota[] quotas = new Quota[types.length];
         for (int i = 0; i < types.length; i++) {
             switch (types[i]) {
-            case FILE:
-                quotas[i] = getFileQuota(folder);
-                break;
-            case STORAGE:
-                quotas[i] = getStorageQuota(folder);
-                break;
-            default:
-                throw FileStorageExceptionCodes.OPERATION_NOT_SUPPORTED.create("Quota " + types[i]);
+                case FILE:
+                    quotas[i] = getFileQuota(folder);
+                    break;
+                case STORAGE:
+                    quotas[i] = getStorageQuota(folder);
+                    break;
+                default:
+                    throw FileStorageExceptionCodes.OPERATION_NOT_SUPPORTED.create("Quota " + types[i]);
             }
         }
         return quotas;
